<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services;

class TipoRecebimento extends CrudWithParameterBag {

    protected $dataSourceName = 'tiporecebimento';
    protected $dataSourceTipo = 'tiporecebimento';
    protected $dataSourceParam = "paramtiporecebimento";
    protected $getTipoAtributo = 'getTipoAtributo';
    /**
     * @var Service\TipoRecebimentoService
     */
    protected $tipoRecebimentoService;
    protected $parameterBagColumns = array('NRORG');

    public function __construct (Manager $dataSourceManager, Services\TipoRecebimentoService $tipoRecebimentoService, ParameterBag $parameterBag) {
        $this->tipoRecebimentoService = $tipoRecebimentoService;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $persistedRows;
            $num = count($dataSet->getRows());
            $row = $dataSet->getRows()[--$num];
            $newRow = $row['__is_new'];       
            $this->configDataSet($dataSet); 
            if (!(array_key_exists("HABILITARVENDAPDV", $row))) {
                if(!$newRow){
                    $dataSet->setDataSourceName('tiporecebimentosave');
                }
                else {
                    $dataSet->setDataSourceName($this->dataSourceTipo);
                }
                $persistedRows = $this->dataSourceManager->persist($dataSet);

                
                $response->addDataSet(new DataSource\DataSet($this->dataSourceTipo, $persistedRows));
                if($newRow)
                    $response->addNotification(new Notification("Dados inseridos com sucesso!", Notification::TYPE_SUCCESS));
                else                        
                    $response->addNotification(new Notification("Dados atualizados com sucesso!", Notification::TYPE_SUCCESS));
            }
            else {

                if($newRow){
                    $row['__is_new'] = true;
                    $dataSet->setDataSourceName($this->dataSourceParam);
                }
                else {
                    $dataSet->setDataSourceName('paramtiporecebimentosave');
                }
                $persistedRows = $this->dataSourceManager->persist($dataSet);

                $response->addDataSet(new DataSource\DataSet($this->dataSourceTipo, $persistedRows));
                if (!$newRow)
                    $response->addNotification(new Notification("Dados atualizados com sucesso!", Notification::TYPE_SUCCESS));
            }            

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $cdtipo = $row['CDTIPORECE'];
                $nrorg = $row['NRORG'];
                $id = 'N';
                $dtult = $row['DTULTATU'];
                $nrult = $row['NRORGULTATU'];
                $cdult = $row['CDOPERULTATU'];
                $this->tipoRecebimentoService->deleta($cdtipo, $nrorg, $id, $dtult, $nrult, $cdult);
            }
            $response->addNotification(new Notification("Dados excluídos com sucesso!", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findCodigo(DTO\Request\DataSet $request, DTO\Response $response) {
        foreach ($request->getDataSet()->getRows() as $row) {
            $response->addDataSet(new DataSet("tiporecebimento", $this->tipoRecebimentoService->findCd($row[0]['CDTIPORECE'], $row[0]['NRORG'])));
        }
    }

    public function findTiposTroco(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $nrorg = $request->getFilterCriteria()->getConditions()[0]['value'];
        $cdtiporece = $request->getFilterCriteria()->getConditions()[1]['value'];
        $data = $this->tipoRecebimentoService->findTiposTroco($nrorg, $cdtiporece);
        $response->addDataSet(new DataSet('tiporecebimentotroco', $data));
    }

    public function saveTiposTroco(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'tiporecebimentotrocosave';
            $i = 0;
            foreach ($request->getDataSet()->getRows() as &$row) {
                if ($row['NRRELTIPOTROCO'] == null){
                    $row['DTINCLUSAO'] = date("d/m/Y H:i:s");
                    $row['__is_new'] = true;
                    $row['NRRELTIPOTROCO'] = $this->tipoRecebimentoService->getTipoTrocoNextId() + $i;
                    $i++;
                } else {
                    $row['DTULTATU'] = date("d/m/Y H:i:s");
                    $row['__is_new'] = false;
                }
            }
            parent::save($request, $response);  
            $this->dataSourceName = 'tiporecebimento';
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function findTipos(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);        
        $dataSet = $this->dataSourceManager->findBy($filterData);
        $data = array();
        $i = 0;
        foreach($dataSet->getRows() as $row) {
            $data[$i] = array(); 
            foreach ($row as $key => $value) {
                if ($key == "IDTIPORECE") {        
                    switch ($value) {
                        case '1': $data[$i][$key] = "Cartão de Crédito";
                            break;
                        case '2': $data[$i][$key] = "Cartão de Débito";
                            break;
                        case '3': $data[$i][$key] = "Cheque";
                            break;
                        case '4': $data[$i][$key] = "Dinheiro";
                            break;
                        case '5': $data[$i][$key] = "Ticket";
                            break;
                        case '6': $data[$i][$key] = "Outras Moedas";
                            break;
                        case '7': $data[$i][$key] = "Contra Vale";
                            break;
                        case '8': $data[$i][$key] = "Convênio";
                            break; 
                        case '9': $data[$i][$key] = "Crédito Pessoal";
                            break;
                        case 'A': $data[$i][$key] = "Débito Consumidor";
                            break;
                        case 'C': $data[$i][$key] = "Pagamento via Site";
                            break;
                    }

                }
                else {
                    $data[$i][$key] = $value;
                }
            }
            $i++;
        }
        $dataSetDef = new DataSource\DataSet($dataSet->getDataSourceName(), $data);
        $response->addDataSet($dataSetDef);
    }

    public function findParams(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceParam);
        $dataSet = $this->dataSourceManager->findBy($filterData);
        $response->addDataSet($dataSet);
    }
    
    private function configDataSet(&$dataSet) {
        foreach ($dataSet->getRows() as $row) {            
            if (!(array_key_exists("HABILITARVENDAPDV", $row))) {
                $this->configTipoRow($row);
            }
            else {
                $this->configParamRow($row);                
            }
        }
    }

    private function configTipoRow(&$row) {
        switch ($row["IDTIPORECE"]) {
            case 'Cartão de Crédito': $row["IDTIPORECE"] = "1";
                break;
            case 'Cartão de Débito': $row["IDTIPORECE"] = "2";
                break;
            case 'Cheque': $row["IDTIPORECE"] = "3";
                break;
            case 'Dinheiro': $row["IDTIPORECE"] = "4";
                break;
            case 'Ticket': $row["IDTIPORECE"] = "5";
                break;
            case 'Outras Moedas': $row["IDTIPORECE"] = "6";
                break;
            case 'Contra Vale': $row["IDTIPORECE"] = "7";
                break;
            case 'Convênio': $row["IDTIPORECE"] = "8";
                break; 
            case 'Crédito Pessoal': $row["IDTIPORECE"] = "9";
                break;
            case 'Débito Consumidor': $row["IDTIPORECE"] = "A";
                break;
            case 'Pagamento via Site': 
                $row["IDTIPORECE"] = "C";
                break;
        }  
        return $row;
    }

    private function configParamRow(&$row) {
        $nrParam = $this->tipoRecebimentoService->checkParam($row['CDTIPORECE']);
        if (! $nrParam)
            $row['NRPARAMETROSTIPORECE'] = $this->tipoRecebimentoService->getNextNumParam();

        else if(empty($row['NRPARAMETROSTIPORECE'])) {
            $row['NRPARAMETROSTIPORECE'] = $nrParam;
        }

        if(!(array_key_exists("__is_new", $row))) {
            $row['__is_new'] = false;
        }

        if ($row["HABILITARVENDAPDV"] == null) {
            $row["HABILITARVENDAPDV"] = "SIM";
        }
        if ($row["PERMITETROCO"] == null) {
            $row["PERMITETROCO"] = "NAO";
        }
        if ($row["GERARPONTUACAOFIDELIDADE"] == null) {
            $row["GERARPONTUACAOFIDELIDADE"] = "SIM";
        }

        $row['NRPARAMETROSTIPORECE'] = (int)$row['NRPARAMETROSTIPORECE'];
           
        return $row;                 
    } 

    private function createParamRow($row) {
        $data = array();

        foreach ($row as $key => $value) {   
            $data[$key] = $value;
        } 
        $data['HABILITARVENDAPDV'] = "SIM"; 
        $data['PERMITETROCO'] = "NAO";  
        $data['GERARPONTUACAOFIDELIDADE'] = "SIM"; 
        $data['NRPARAMETROSTIPORECE'] = $this->tipoRecebimentoService->getNextNumParam(); 
        $data['__is_new'] = true;       
        unset($data['NMTIPORECE']);
        unset($data['IDATIVO']);
        unset($data['IDTIPORECE']); 
        unset($data['DSIMPFISCAL']); 
        unset($data['IDSOLIQTPARC']); 
        unset($data['QTMAXPARC']); 
        unset($data['VRMINPARC']);  
              
        return $data;                    
    }

    public function findAtriTipoRecebimento(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName('relatributotiporece');
        $dataSet = $this->dataSourceManager->findBy($filterData);
        $response->addDataSet($dataSet);
    }

    public function saveAtriTipoRecebimento(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName('saverelatributotiporece');
            $rows = $dataSet->getRows();
            foreach ($rows as $row) {
                if ($row['__is_new']){
                    $row['NRRELAATRIBUTOTIPORECE'] = $this->tipoRecebimentoService->getNextId();
                }
            }
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            if($request->getDataSet()->getRows()[0]['IDATIVO'] == 'N'){
                $response->addNotification(new Notification("Excluído com sucesso!", Notification::TYPE_SUCCESS));
            } else {
                $response->addNotification(new Notification("Salvo com sucesso!", Notification::TYPE_SUCCESS));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getTipoAtributo(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->getTipoAtributo;
        parent::find($request, $response);
    }
}